function [X W H opt] = genSynthSimplexData(K, M, N, sigma2, mix, opt);

% Generate synthetic data within simplex, i.e. X,W,H > 0 and sum(H) = 1
%
% INPUT
% - K           : No. vertices
% - M           : No. dimensions
% - N           : No. samples
% - sigma2      : Noise level, AWGN.
% - mix         : Mixing profile
% - opt
%   .W          : Init W
%   .randState  : Initial state for random generator
%   .doPlot     : Plot or not
%
% OUTPUT
% - X           : Output samples (MxN)
% - W           : Vertices/endmembers (MxK)
% - H           : Fractional abundances (KxN)
%
% Copyright 2009, Morten Arngren (ma@imm.dtu.dk)
%
% V1.0 March 2010   : Initial issue.
%

try, sigma2;        catch sigma2        = 0;    end   %
try, mix;           catch mix           = [];   end   %
try, opt.W.init;    catch opt.W.init    = 0;    end   %
try, opt.randState; catch opt.randState = [];   end   %
try, opt.doPlot;    catch opt.doPlot    = 0;    end   %

% Initialize
if isempty(mix), mix = ones(K,1); end

if opt.randState
  rand('state', opt.randState);
end

if M == 0, M = size(opt.W,1); end

% Generate random W
if ~opt.W.init
  W = rand(M,K);
else
  W = opt.W;
end

% Generate random H, rejection sampling
H  = zeros(K,N);
NN = 0;

while NN < N
  Htmp            = -log(rand(K,N));
  Htmp            = Htmp ./ (ones(K,1)*sum(Htmp));
  id              = Htmp(1,:)>mix(1) | Htmp(2,:)>mix(2) | Htmp(3,:)>mix(3);
  Htmp(:,id)      = [];
  cp              = min(size(Htmp,2),N-NN);
  H(:,NN+1:NN+cp) = Htmp(:,1:cp);
  NN              = NN + size(Htmp,2);
end

% Factor model with AWGN
X = W*H + sqrt(sigma2)*randn(M,N);

if opt.doPlot
  [U D mu UW] = PCA(W);
  UX       = U'*X;
  figure(gcf),
  plot(UX(1,:), UX(2,:), '.r', 'LineWidth', 2); hold on;
  plot(UW(1,:), UW(2,:), 'xk', 'LineWidth', 3, 'MarkerSize', 10);
  hold off; grid on;
  figure(gcf+1);
  plot3(UX(1,:), UX(2,:), UX(3,:), '.r', 'LineWidth', 2); hold on;
  plot3(UW(1,:), UW(2,:), UW(3,:), 'xk', 'LineWidth', 3, 'MarkerSize', 10);
  hold off; grid on; axis equal;
end

opt.W.true = W;

return
